# -*- coding: utf-8 -*-
import json
import sys
from datetime import datetime
from time import time
import os
import io
import argparse
import pandas as pd
import numpy as np
import math
from loguru import logger
import warnings
#import openpyxl
import PyPDF2 
import docx
import jieba
from sklearn.cluster import KMeans, MiniBatchKMeans
from sklearn.decomposition import PCA
from sklearn.decomposition import TruncatedSVD
from sklearn.decomposition import NMF, LatentDirichletAllocation, MiniBatchNMF
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.feature_extraction.text import HashingVectorizer
from sklearn.feature_extraction.text import TfidfTransformer
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.feature_extraction import DictVectorizer
from sklearn.pipeline import make_pipeline
from sklearn.preprocessing import Normalizer
from sklearn import metrics
import matplotlib.pyplot as plt
from matplotlib.font_manager import FontProperties
import traceback
import pyLDAvis
import pyLDAvis.lda_model
#from sklearn.datasets import fetch_20newsgroups


"""
使用下面的命令安装需要的模块：
pip install -U pandas openpyxl loguru python-docx pyinstaller PyPDF2 xlrd scikit-learn jieba matplotlib pyLDAvis
"""

# 当前版本
current_version = 'GooSeeker_Cluster 2.0.7'

# 设置系统编码为UTF-8
os.environ['PYTHONIOENCODING'] = 'utf-8'
sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding='utf-8')
sys.stderr = io.TextIOWrapper(sys.stderr.buffer, encoding='utf-8')

# 返回当前版本号
def get_version():
    return current_version

# 判断当前操作系统 
def get_os():
    if os.name == 'nt':
        return 'windows'
    elif os.name == 'posix':
        if os.uname().sysname == 'Darwin':
            return 'mac'
        else:
            return 'linux'
    else:
        return 'unknown'

# 过滤掉FutureWarning和UserWarning 
try:
    pd.set_option('future.no_silent_downcasting', True)
except Exception as e:
    #logger.info('pd.set_option:' + str(e))   
    logger.error(f"{traceback.format_exc()}")    
warnings.filterwarnings('ignore', category=UserWarning, message='Workbook contains no default style')
#warnings.filterwarnings('ignore', category=ConvergenceWarning)
#warnings.filterwarnings('ignore', module='sklearn')
warnings.simplefilter(action='ignore', category=FutureWarning)
warnings.filterwarnings('ignore', category=RuntimeWarning)
# jieba.setLogLevel('CRITICAL')
# logger.getLogger('sklearn').setLevel('CRITICAL')

# 日志级别
LOG_LEVEL = 'INFO'  # ['TRACE', 'DEBUG', 'INFO', 'SUCCESS', 'WARNING', 'ERROR', 'CRITICAL']

# ###############################################
# 输入导入相关
# ###############################################

# 目标文件名
FILE_ORIGIN_DATA_XLSX = '原始数据表.xlsx'
FILE_ORIGIN_DATA_CSV = '原始数据表.csv'
FILE_SEG_EFFECT_XLSX = '分词效果表.xlsx'
FILE_SEG_EFFECT_CSV = '分词效果表.xlsx'
FILE_WORD_CHOICE_MATCH_XLSX = '选词匹配表.xlsx'
FILE_WORD_CHOICE_MATCH_CSV = '选词匹配表.csv'

# 聚类目录名
GOOSEEKERTAG_DIR = 'GooSeekerTag'
DEFAULT_CLUSTER_DIR = os.path.join('GooSeekerTag','local')
DIR_CLUSTER = os.path.join('cluster')
FILE_DATA_JSON = os.path.join(DIR_CLUSTER,'data.json')
FILE_CLUSTER_JSON = os.path.join(DIR_CLUSTER,'result.json')
DIR_RAW = os.path.join(DIR_CLUSTER,'raw')
DIR_IMG = os.path.join(DIR_CLUSTER,'img')
DIR_PROCESSED = os.path.join(DIR_CLUSTER,'processed')
# lda目录名
DIR_LDA = os.path.join('lda')
FILE_DATA_JSON_LDA = os.path.join(DIR_LDA,'data.json')
FILE_LDA_JSON = os.path.join(DIR_LDA,'result.json')
DIR_RAW_LDA = os.path.join(DIR_LDA,'raw')
DIR_IMG_LDA = os.path.join(DIR_LDA,'img')
DIR_PROCESSED_LDA = os.path.join(DIR_LDA,'processed')
# 日志文件
#DIR_LOG = os.path.join(DIR_CLUSTER,'log')
#FILE_LOG = os.path.join(DIR_CLUSTER,'log','cluster.log')
DIR_LOG = os.path.join(os.path.expanduser('~'),GOOSEEKERTAG_DIR, 'log')
FILE_LOG = os.path.join(DIR_LOG,'cluster.log')

# excel列名
# 原始数据excel必须包含的列名
COLUMNS_FILE_ORIGIN_DATA_XLSX = ['序号', '正文']
# 分词效果表excel必须包含的列名
COLUMNS_FILE_SEG_EFFECT_XLSX = ['序号', '分词数据']
# 选词匹配表excel必须包含的列名
COLUMNS_FILE_WORD_CHOICE_MATCH_XLSX = ['序号', '打标词']
# supported input file extension
FILE_EXTENSION_ORIGIN_DATA = ['xls','xlsx','csv','docx','pdf','txt']
FILE_EXTENSION_SEG_EFFECT = ['xlsx']
FILE_EXTENSION_WORD_CHOICE_MATCH = ['xlsx']
FILE_EXTENSION_ALL = ['xls','xlsx','csv','docx','pdf','txt']
# FILE_EXTENSION = ['xls','xlsx','csv','docx','pdf','txt']

# data.json样例
DATA_SAMPLE = """{"原始数据表": {"modify": "", "total": 0}, "分词效果表": {"modify": "","total": 0},"选词匹配表": {"modify": "","total": 0}}"""

# exit code
exit_code = 0



# 获取当前时间的字符串，格式“YYYY-MM-DDDD HH:MM”
def get_time():
    return datetime.now().strftime("%Y-%m-%d %H:%M")

# 把路径字符串按unix格式标准化    
def unix_path(inputpath):
    #return inputpath.replace('\\','/')
    return inputpath

# 获取data.json文件中指定类型的modify时间
def get_value_from_data_json(file_path, file_type, key):
    data_json_sample = json.loads(DATA_SAMPLE)
    return_value = ""
    if os.path.exists(file_path):
        try:
            with open(file_path, 'r', encoding='utf-8') as file_read:
                data = json.load(file_read)
        # 如果不是合法的json格式，则从DATA_SAMPLE文件load
        except json.JSONDecodeError:
            logger.error(f"{traceback.format_exc()}")
            logger.info(f"json文件：{file_path} 格式非法，加载DATA_SAMPLE")
            data = data_json_sample
    else:
        data = data_json_sample
    if file_type == 1:
        logger.debug("key_top = FILE_ORIGIN_DATA_XLSX.split('.')[0]")
        key_top = FILE_ORIGIN_DATA_XLSX.split('.')[0]
    elif file_type == 2:
        logger.debug("key_top = FILE_SEG_EFFECT_XLSX.split('.')[0]")
        key_top = FILE_SEG_EFFECT_XLSX.split('.')[0]
    elif file_type == 3:
        logger.debug("key_top = FILE_WORD_CHOICE_MATCH_XLSX.split('.')[0]")
        key_top = FILE_WORD_CHOICE_MATCH_XLSX.split('.')[0]
    if key_top in data.keys():
        if key in data[key_top]:
            return_value = data[key_top][key]
    return return_value
    
   
# 更新data.json
def update_json(file_path, file_type, modify, total):
    data_json_sample = json.loads(DATA_SAMPLE)
    # 检查data.json是否存在
    if os.path.exists(file_path):
        try:
            with open(file_path, 'r', encoding='utf-8') as file_read:
                data = json.load(file_read)
        # 如果不是合法的json格式，则从DATA_SAMPLE文件load
        except json.JSONDecodeError:
            logger.error(f"{traceback.format_exc()}")
            logger.info(f"json文件：{file_path} 格式非法，加载DATA_SAMPLE")
            data = data_json_sample
    else:
        data = data_json_sample
    # 检查json的key是否存在 
    for key in data_json_sample.keys():
        if key not in data:
            data[key] = json.loads('{"modify": "", "total": 0}')
        if "modify" not in data[key]:
            data[key]["modify"]=""
        if "total" not in data[key]:
            data[key]["total"]=0
    # 更新json数据
    if file_type == 1:
        data["原始数据表"]["modify"] = modify
        data["原始数据表"]["total"] = total
    elif file_type == 2:
        data["分词效果表"]["modify"] = modify
        data["分词效果表"]["total"] = total
    elif file_type == 3:
        data["选词匹配表"]["modify"] = modify
        data["选词匹配表"]["total"] = total
    with open(file_path, 'w', encoding='utf-8') as file_write:
        json.dump(data, file_write, ensure_ascii=False, indent=4)

# tag任务目录初始化
def tag_dir_init(tag_dir, tag_name, datatype='cluster'):
    # 创建cluster/log日志目录
    #log_dir = tag_dir + "/" + tag_name + DIR_LOG
    #log_dir = os.path.join(tag_dir , tag_name , DIR_LOG)
    log_dir = DIR_LOG
    if not os.path.exists(log_dir):
        # 如果目录不存在，则创建目录
        os.makedirs(log_dir)
    if datatype == 'cluster':
        # 创建cluster/raw目录
        #raw_dir = tag_dir + "/" + tag_name + DIR_RAW
        raw_dir = os.path.join(tag_dir , tag_name , DIR_RAW)
        if not os.path.exists(raw_dir):
            # 如果目录不存在，则创建目录
            os.makedirs(raw_dir)
        # 创建cluster/processed目录
        #processed_dir = tag_dir + "/" + tag_name + DIR_PROCESSED
        processed_dir = os.path.join(tag_dir , tag_name , DIR_PROCESSED)
        if not os.path.exists(processed_dir):
            # 如果目录不存在，则创建目录
            os.makedirs(processed_dir)
        # 创建cluster/img目录
        #img_dir = tag_dir + "/" + tag_name + DIR_IMG
        img_dir = os.path.join(tag_dir , tag_name , DIR_IMG)
        if not os.path.exists(img_dir):
            # 如果目录不存在，则创建目录
            os.makedirs(img_dir)
    else:
        # 创建lda/raw目录
        #raw_dir = tag_dir + "/" + tag_name + DIR_RAW
        raw_dir = os.path.join(tag_dir , tag_name , DIR_RAW_LDA)
        if not os.path.exists(raw_dir):
            # 如果目录不存在，则创建目录
            os.makedirs(raw_dir)
        # 创建cluster/processed目录
        #processed_dir = tag_dir + "/" + tag_name + DIR_PROCESSED
        processed_dir = os.path.join(tag_dir , tag_name , DIR_PROCESSED_LDA)
        if not os.path.exists(processed_dir):
            # 如果目录不存在，则创建目录
            os.makedirs(processed_dir)
        # 创建cluster/img目录
        #img_dir = tag_dir + "/" + tag_name + DIR_IMG
        img_dir = os.path.join(tag_dir , tag_name , DIR_IMG_LDA)
        if not os.path.exists(img_dir):
            # 如果目录不存在，则创建目录
            os.makedirs(img_dir)    
        
# 生成空的excel文件和csv文件
def create_new_excel(filename, file_type):
    if file_type == 1:
        # 创建一个空的DataFrame
        df = pd.DataFrame(columns=COLUMNS_FILE_ORIGIN_DATA_XLSX)
        # 保存为Excel文件
        df.to_excel(filename, index=False)
        df.to_csv(filename.replace('.xlsx','.csv'), index=False, encoding='utf-8')
    elif file_type == 2:
        # 创建一个空的DataFrame
        df = pd.DataFrame(columns=COLUMNS_FILE_SEG_EFFECT_XLSX)
        # 保存为Excel文件
        df.to_excel(filename, index=False)
        df.to_csv(filename.replace('.xlsx','.csv'), index=False, encoding='utf-8')
    elif file_type == 3:
        # 创建一个空的DataFrame
        df = pd.DataFrame(columns=COLUMNS_FILE_WORD_CHOICE_MATCH_XLSX)
        # 保存为Excel文件
        df.to_excel(filename, index=False)
        df.to_csv(filename.replace('.xlsx','.csv'), index=False, encoding='utf-8')

# 读取pdf文本内容        
def get_pdf_content(file_path):     
    text = ""     
    with open(file_path, "rb") as file:         
        pdf = PyPDF2.PdfReader(file)         
        for page_num in range(len(pdf.pages)): 
            page_obj = pdf.pages[page_num]
            page_content = page_obj.extract_text()
            text += page_content     
    return text

# 读取docx文本内容 
# 需安装python-docx: pip install python-docx
def get_docx_content(file_path): 
    text = ""
    doc = docx.Document(file_path)
    for para in doc.paragraphs:
        text += para.text 
    return text
    
# 原始数据表导入        
def import_file_origin_data(desfilename, file_type, srcfilename):
    # 原始数据表不存在则创建一个
    if not os.path.exists(desfilename):
        create_new_excel(desfilename, file_type)
    df1 = pd.read_excel(desfilename)
    srcfile_extension = os.path.basename(srcfilename).split('.')[-1]
    logger.debug(f"{srcfilename} extention：{srcfile_extension}")
    if srcfile_extension == 'xlsx':
    # 原始数据表是.xlsx
        df = pd.read_excel(srcfilename)
        flag_columns_exist = True
        for column in COLUMNS_FILE_ORIGIN_DATA_XLSX:
            if column not in df.columns:
                flag_columns_exist = False
        if flag_columns_exist:
            #df_to_append = df[COLUMNS_FILE_ORIGIN_DATA_XLSX]
            df_to_append = df
            if not df1.empty:
                df_to_save = pd.concat([df1, df_to_append], axis=0)
            else:
                df_to_save = df_to_append.copy()
            df_to_save.to_excel(desfilename, index=False)
            df_to_save.to_csv(desfilename.replace('.xlsx','.csv'), header=True, index=False)
            return len(df_to_save)
        else:
            logger.error(f"文件{srcfilename}没有以下字段：{COLUMNS_FILE_ORIGIN_DATA_XLSX}")
            return 0
    # 原始数据表是.xls
    elif srcfile_extension == 'xls':
        df = pd.read_excel(srcfilename)
        flag_columns_exist = True
        for column in COLUMNS_FILE_ORIGIN_DATA_XLSX:
            if column not in df.columns:
                flag_columns_exist = False
        if flag_columns_exist:
            #df_to_append = df[COLUMNS_FILE_ORIGIN_DATA_XLSX]
            df_to_append = df
            if not df1.empty:
                df_to_save = pd.concat([df1, df_to_append], axis=0)
            else:
                df_to_save = df_to_append.copy()
            df_to_save.to_excel(desfilename, index=False)
            df_to_save.to_csv(desfilename.replace('.xlsx','.csv'), header=True, index=False)
            return len(df_to_save)
        else:
            logger.error(f"文件{srcfilename}没有以下字段：{COLUMNS_FILE_ORIGIN_DATA_XLSX}")
            return 0
    # 原始数据表是.csv
    elif srcfile_extension == 'csv':
        df = pd.read_csv(srcfilename)
        flag_columns_exist = True
        for column in COLUMNS_FILE_ORIGIN_DATA_XLSX:
            if column not in df.columns:
                flag_columns_exist = False
        if flag_columns_exist:
            #df_to_append = df[COLUMNS_FILE_ORIGIN_DATA_XLSX]
            df_to_append = df
            if not df1.empty:
                df_to_save = pd.concat([df1, df_to_append], axis=0)
            else:
                df_to_save = df_to_append.copy()
            df_to_save.to_excel(desfilename, index=False)
            df_to_save.to_csv(desfilename.replace('.xlsx','.csv'), header=True, index=False)
            return len(df_to_save)
        else:
            logger.error(f"文件{srcfilename}没有以下字段：{COLUMNS_FILE_ORIGIN_DATA_XLSX}")
            return 0        
    # 原始数据表是.txt
    elif srcfile_extension == 'txt':
        with open(srcfilename, 'r', encoding='utf-8') as file:
            content = file.read()
        if content:
            df_to_append = pd.DataFrame([[os.path.basename(srcfilename),content]], columns=COLUMNS_FILE_ORIGIN_DATA_XLSX)
            df_to_save = pd.concat([df1, df_to_append], axis=0)
            df_to_save.to_excel(desfilename, index=False)
            df_to_save.to_csv(desfilename.replace('.xlsx','.csv'), header=True, index=False)
            return 1
        else:
            return 0
    # 原始数据表是.pdf
    elif srcfile_extension == 'pdf':
        content = get_pdf_content(srcfilename)
        if content:
            df_to_append = pd.DataFrame([[os.path.basename(srcfilename),content]], columns=COLUMNS_FILE_ORIGIN_DATA_XLSX)
            df_to_save = pd.concat([df1, df_to_append], axis=0)
            df_to_save.to_excel(desfilename, index=False)
            df_to_save.to_csv(desfilename.replace('.xlsx','.csv'), header=True, index=False)
            return 1
        else:
            return 0
    # 原始数据表是.docx
    elif srcfile_extension == 'docx':
        content = get_docx_content(srcfilename)
        if content:
            df_to_append = pd.DataFrame([[os.path.basename(srcfilename),content]], columns=COLUMNS_FILE_ORIGIN_DATA_XLSX)
            df_to_save = pd.concat([df1, df_to_append], axis=0)
            df_to_save.to_excel(desfilename, index=False)
            df_to_save.to_csv(desfilename.replace('.xlsx','.csv'), header=True, index=False)
            return 1
        else:
            return 0
    # 原始数据表是.doc
    elif srcfile_extension == 'doc':
        logger.error(f"input file extension:{srcfile_extension} not supported yet!!!")
        return 0
    else:
        logger.error(f"input file extension:{srcfile_extension} not supported yet!!!")
        return 0
            
# 分词效果表导入        
def import_file_seg_effect(desfilename, file_type, srcfilename):
    # 分词数据表不存在则创建一个
    if not os.path.exists(desfilename):
        create_new_excel(desfilename, file_type)
    df1 = pd.read_excel(desfilename)
    df = pd.read_excel(srcfilename)
    flag_columns_exist = True
    for column in COLUMNS_FILE_SEG_EFFECT_XLSX:
        if column not in df.columns:
            flag_columns_exist = False
    if flag_columns_exist:
        #df_to_append = df[COLUMNS_FILE_SEG_EFFECT_XLSX]
        df_to_append = df
        df_to_save = pd.concat([df1, df_to_append], axis=0)
        df_to_save.to_excel(desfilename, index=False)
        df_to_save.to_csv(desfilename.replace('.xlsx','.csv'), header=True, index=False)
        return len(df_to_save)
    else:
        logger.error(f"文件{srcfilename}没有以下字段：{COLUMNS_FILE_SEG_EFFECT_XLSX}")
        return 0        
 
# 选词匹配表导入 
def import_file_word_choice_match(desfilename, file_type, srcfilename):
    # 选词匹配表不存在则创建一个
    if not os.path.exists(desfilename):
        create_new_excel(desfilename, file_type)
    df1 = pd.read_excel(desfilename)
    df = pd.read_excel(srcfilename)
    flag_columns_exist = True
    for column in COLUMNS_FILE_WORD_CHOICE_MATCH_XLSX:
        if column not in df.columns:
            flag_columns_exist = False
    if flag_columns_exist:
        #df_to_append = df[COLUMNS_FILE_WORD_CHOICE_MATCH_XLSX]
        df_to_append = df
        df_to_save = pd.concat([df1, df_to_append], axis=0)
        df_to_save.to_excel(desfilename, index=False)
        df_to_save.to_csv(desfilename.replace('.xlsx','.csv'), header=True, index=False)
        return len(df_to_save)
    else:
        logger.error(f"文件{srcfilename}没有以下字段：{COLUMNS_FILE_WORD_CHOICE_MATCH_XLSX}")
        return 0  

# 检查excel文件对应的数据表类型
def check_data_type(srcfilename):
    srcfile_extension = os.path.basename(srcfilename).split('.')[-1]
    logger.info(f"{srcfilename} extention：{srcfile_extension}")
    if srcfile_extension == 'xlsx':
        logger.debug(f"before pd.read_excel")
        df = pd.read_excel(srcfilename)
        logger.debug(f"after pd.read_excel")
        # flag_column_sn_exist = False
        # flag_column_ORIGIN_DATA_exist = False
        # flag_column_SEG_EFFECT_exist = False
        # flag_column_WORD_CHOICE_MATCH_exist = False
        if COLUMNS_FILE_ORIGIN_DATA_XLSX[0] in df.columns:
            if COLUMNS_FILE_ORIGIN_DATA_XLSX[1] in df.columns:
                return 1
            elif COLUMNS_FILE_SEG_EFFECT_XLSX[1] in df.columns:
                return 2
            elif COLUMNS_FILE_WORD_CHOICE_MATCH_XLSX[1] in df.columns:
                return 3
    return 104
    
# ###############################################
# 聚类相关
# ###############################################

# 聚类使用的列名
COLUMN_CLUSTER = '已分词内容'

# 特征抽取名称
FEATURE = ["feature_01", "feature_freq", "feature_tfidf"] 

# 源数据名称
SOURCE = ["原始数据表", "分词效果表", "选词匹配表"] 

# 停用词表
STOPWORDS = ['$', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '?', '_', '“', '”', '、', '。', '《', '》', '一', '一些', '一何', '一切', '一则', '一方面', '一旦', '一来', '一样', '一般', '一转眼', '万一', '上', '上下', '下', '不', '不仅', '不但', '不光', '不单', '不只', '不外乎', '不如', '不妨', '不尽', '不尽然', '不得', '不怕', '不惟', '不成', '不拘', '不料', '不是', '不比', '不然', '不特', '不独', '不管', '不至于', '不若', '不论', '不过', '不问', '与', '与其', '与其说', '与否', '与此同时', '且', '且不说', '且说', '两者', '个', '个别', '临', '为', '为了', '为什么', '为何', '为止', '为此', '为 着', '乃', '乃至', '乃至于', '么', '之', '之一', '之所以', '之类', '乌乎', '乎', '乘', '也', '也好', '也罢', '了', '二来', '于', '于是', '于是乎', ' 云云', '云尔', '些', '亦', '人', '人们', '人家', '什么', '什么样', '今', '介于', '仍', '仍旧', '从', '从此', '从而', '他', '他人', '他们', '以', '以 上', '以为', '以便', '以免', '以及', '以故', '以期', '以来', '以至', '以至于', '以致', '们', '任', '任何', '任凭', '似的', '但', '但凡', '但是', '何', '何以', '何况', '何处', '何时', '余外', '作为', '你', '你们', '使', '使得', '例如', '依', '依据', '依照', '便于', '俺', '俺们', '倘', '倘使', '倘或', '倘然', '倘若', '借', '假使', '假如', '假若', '傥然', '像', '儿', '先不先', '光是', '全体', '全部', '兮', '关于', '其', '其一', '其中', '其二', ' 其他', '其余', '其它', '其次', '具体地说', '具体说来', '兼之', '内', '再', '再其次', '再则', '再有', '再者', '再者说', '再说', '冒', '冲', '况且', ' 几', '几时', '凡', '凡是', '凭', '凭借', '出于', '出来', '分别', '则', '则甚', '别', '别人', '别处', '别是', '别的', '别管', '别说', '到', '前后', ' 前此', '前者', '加之', '加以', '即', '即令', '即使', '即便', '即如', '即或', '即若', '却', '去', '又', '又及', '及', '及其', '及至', '反之', '反而', '反过来', '反过来说', '受到', '另', '另一方面', '另外', '另悉', '只', '只当', '只怕', '只是', '只有', '只消', '只要', '只限', '叫', '叮咚', '可', '可以', '可是', '可见', '各', '各个', '各位', '各种', '各自', '同', '同时', '后', '后者', '向', '向使', '向着', '吓', '吗', '否则', '吧', '吧哒', '吱', '呀', '呃', '呕', '呗', '呜', '呜呼', '呢', '呵', '呵呵', '呸', '呼哧', '咋', '和', '咚', '咦', '咧', '咱', '咱们', '咳', '哇', '哈', '哈哈', '哉', '哎', '哎呀', '哎哟', '哗', '哟', '哦', '哩', '哪', '哪个', '哪些', '哪儿', '哪天', '哪年', '哪怕', '哪样', '哪边', '哪里', '哼', '哼唷', '唉', '唯有', '啊', '啐', '啥', '啦', '啪达', '啷当', '喂', '喏', '喔唷', '喽', '嗡', '嗡嗡', '嗬', '嗯', '嗳', '嘎', '嘎登', '嘘', '嘛', '嘻', '嘿', '嘿嘿', '因', '因为', '因了', '因此', '因着', '因而', '固然', '在', '在下', '在于', '地', '基于', '处在', '多', '多么', '多少', '大', '大家', '她', '她们', '好', '如', '如上', '如上所述', '如下', '如何', '如其', '如同', '如是', '如果', '如此', '如若', '始而', '孰料', '孰知', '宁', '宁可', '宁愿', '宁肯', '它', '它们', '对', '对于', '对待', '对方', '对比', '将', '小', '尔', '尔后', '尔尔', '尚且', '就', '就是', '就是了', '就是说', '就算', '就要', '尽', ' 尽管', '尽管如此', '岂但', '己', '已', '已矣', '巴', '巴巴', '并', '并且', '并非', '庶乎', '庶几', '开外', '开始', '归', '归齐', '当', '当地', '当然', '当着', '彼', '彼时', '彼此', '往', '待', '很', '得', '得了', '怎', '怎么', '怎么办', '怎么样', '怎奈', '怎样', '总之', '总的来看', '总的来说', '总的说来', '总而言之', '恰恰相反', '您', '惟其', '慢说', '我', '我们', '或', '或则', '或是', '或曰', '或者', '截至', '所', '所以', '所在', '所幸', '所 有', '才', '才能', '打', '打从', '把', '抑或', '拿', '按', '按照', '换句话说', '换言之', '据', '据此', '接着', '故', '故此', '故而', '旁人', '无', ' 无宁', '无论', '既', '既往', '既是', '既然', '时候', '是', '是以', '是的', '曾', '替', '替代', '最', '有', '有些', '有关', '有及', '有时', '有的', ' 望', '朝', '朝着', '本', '本人', '本地', '本着', '本身', '来', '来着', '来自', '来说', '极了', '果然', '果真', '某', '某个', '某些', '某某', '根据', '欤', '正值', '正如', '正巧', '正是', '此', '此地', '此处', '此外', '此时', '此次', '此间', '毋宁', '每', '每当', '比', '比及', '比如', '比方', '没奈何', '沿', '沿着', '漫说', '焉', '然则', '然后', '然而', '照', '照着', '犹且', '犹自', '甚且', '甚么', '甚或', '甚而', '甚至', '甚至于', '用', '用来', '由', '由于', '由是', '由此', '由此可见', '的', '的确', '的话', '直到', '相对而言', '省得', '看', '眨眼', '着', '着呢', '矣', '矣乎', '矣哉', '离', '竟而', '第', '等', '等到', '等等', '简言之', '管', '类如', '紧接着', '纵', '纵令', '纵使', '纵然', '经', '经过', '结果', '给', '继之', '继后', '继 而', '综上所述', '罢了', '者', '而', '而且', '而况', '而后', '而外', '而已', '而是', '而言', '能', '能否', '腾', '自', '自个儿', '自从', '自各儿', ' 自后', '自家', '自己', '自打', '自身', '至', '至于', '至今', '至若', '致', '般的', '若', '若夫', '若是', '若果', '若非', '莫不然', '莫如', '莫若', ' 虽', '虽则', '虽然', '虽说', '被', '要', '要不', '要不是', '要不然', '要么', '要是', '譬喻', '譬如', '让', '许多', '论', '设使', '设或', '设若', '诚 如', '诚然', '该', '说来', '诸', '诸位', '诸如', '谁', '谁人', '谁料', '谁知', '贼死', '赖以', '赶', '起', '起见', '趁', '趁着', '越是', '距', '跟', '较', '较之', '边', '过', '还', '还是', '还有', '还要', '这', '这一来', '这个', '这么', '这么些', '这么样', '这么点儿', '这些', '这会儿', '这儿', '这就是说', '这时', '这样', '这次', '这般', '这边', '这里', '进而', '连', '连同', '逐步', '通过', '遵循', '遵照', '那', '那个', '那么', '那么些', '那么 样', '那些', '那会儿', '那儿', '那时', '那样', '那般', '那边', '那里', '都', '鄙人', '鉴于', '针对', '阿', '除', '除了', '除外', '除开', '除此之外', '除非', '随', '随后', '随时', '随着', '难道说', '非但', '非徒', '非特', '非独', '靠', '顺', '顺着', '首先', '！', '，', '：', '；', '？']

# 特征提取
def feature_extraction(file_type, src_df, scale, para_dict):
    # scale的取值有3种，1：“0/1” 2：词频 3：tf-idf
    if scale == 1:
        #vectorizer = CountVectorizer(max_features=100)
        vectorizer = CountVectorizer(max_features=para_dict["CountVectorizer"]["max_features"])
        matrix_x = vectorizer.fit_transform(src_df[COLUMN_CLUSTER].values.astype('U'))
        return vectorizer, matrix_x
    elif scale == 2:
        # vectorizer = CountVectorizer(max_df=0.5, min_df=0.005, stop_words=STOPWORDS, max_features=100)
        vectorizer = CountVectorizer(max_features=para_dict["CountVectorizer"]["max_features"])
        matrix_x = vectorizer.fit_transform(src_df[COLUMN_CLUSTER].values.astype('U'))
        return vectorizer, matrix_x
    elif scale == 3:
        if file_type == 3:
            # 把打标词转换成list
            logger.debug("src_df['voc']=src_df['打标词'].apply(lambda x:x.split(','))")
            src_df['voc']=src_df['打标词'].apply(lambda x:x.split(','))
            # 生成打标词集合
            voc_set = set([])
            for item in src_df['voc']:
                voc_set.update(item)
            logger.debug(f"voc_set的元素个数：{len(voc_set)}")
            vectorizer = TfidfVectorizer(max_df=para_dict["TfidfVectorizer"]["max_df"], min_df=para_dict["TfidfVectorizer"]["min_df"], analyzer='char', vocabulary=voc_set,  ngram_range=(2, 4))
            # vectorizer = TfidfVectorizer(stop_words=STOPWORDS,vocabulary=voc_set, analyzer='char', ngram_range=(2, 4))
            matrix_x = vectorizer.fit_transform(src_df[COLUMN_CLUSTER].values.astype('U'))
            # 最多只使用500个特征
            n_compon = matrix_x.shape[1]
            if n_compon > para_dict["TruncatedSVD"]["n_components"]:
                n_compon = para_dict["TruncatedSVD"]["n_components"]
            lsa = make_pipeline(TruncatedSVD(n_components=n_compon), Normalizer(copy=False))
            X_lsa = lsa.fit_transform(matrix_x)
            return vectorizer, X_lsa, lsa[0]
        else:
            vectorizer = TfidfVectorizer(max_df=para_dict["TfidfVectorizer"]["max_df"], min_df=para_dict["TfidfVectorizer"]["min_df"], stop_words=para_dict["TfidfVectorizer"]["stop_words"])
            #vectorizer = TfidfVectorizer(max_df=0.5, min_df=0.005, stop_words=STOPWORDS)
            matrix_x = vectorizer.fit_transform(src_df[COLUMN_CLUSTER].values.astype('U'))
            # 最多只使用500个特征
            n_compon = matrix_x.shape[1]
            if n_compon > para_dict["TruncatedSVD"]["n_components"]:
                n_compon = para_dict["TruncatedSVD"]["n_components"]
            lsa = make_pipeline(TruncatedSVD(n_components=n_compon), Normalizer(copy=False))
            X_lsa = lsa.fit_transform(matrix_x)
            return vectorizer, X_lsa, lsa[0]


def save_cluster_result(cluster_result_json, file_cluster_json, file_type, scale, category):
    logger.debug(f"file_cluster_json文件：0")
    data_json_sample = {}
    # 检查data.json是否存在
    if os.path.exists(file_cluster_json):
        try:
            with open(file_cluster_json, 'r', encoding='utf-8') as file_read:
                data = json.load(file_read)
        # 如果不是合法的json格式，则从DATA_SAMPLE文件load
        except json.JSONDecodeError:
            logger.error(f"{traceback.format_exc()}")
            logger.info(f"file_cluster_json文件：{file_cluster_json} 格式非法，使用空的json")
            data = data_json_sample
    else:
        data = data_json_sample
    # 确定要更新的key
    key1 = SOURCE[file_type - 1]
    key2 = FEATURE[scale - 1]
    key3 = 'class_' + str(category)
    # 检查并更新json
    logger.debug(f"file_cluster_json文件：1")
    if key1 not in data.keys():
        data[key1] = {}
    logger.debug(f"file_cluster_json文件：2")
    if key2 not in data[key1].keys():
        data[key1][key2] = {}
    logger.debug(f"file_cluster_json文件：3")
    if key3 not in data[key1][key2].keys():
        data[key1][key2][key3] = {} 
    data[key1][key2][key3] = cluster_result_json
    logger.debug(f"file_cluster_json文件：4")
    with open(file_cluster_json, 'w', encoding='utf-8') as file_write:
        json.dump(data, file_write, ensure_ascii=False, indent=4)


        
def save_wordset_result(cluster_result_json, file_wordset_json, file_type, scale, category):
    logger.debug(f"file_cluster_json文件：0")
    data_json_sample = {}
    # 检查data.json是否存在
    if os.path.exists(file_wordset_json):
        try:
            with open(file_wordset_json, 'r', encoding='utf-8') as file_read:
                data = json.load(file_read)
        # 如果不是合法的json格式，则从DATA_SAMPLE文件load
        except json.JSONDecodeError:
            logger.error(f"{traceback.format_exc()}")
            logger.info(f"file_wordset_json{file_wordset_json} 格式非法，使用空的json")
            data = data_json_sample
    else:
        data = data_json_sample
    # 确定要更新的key
    key1 = SOURCE[file_type - 1]
    key2 = FEATURE[scale - 1]
    key3 = 'class_' + str(category)
    # 检查并更新json
    logger.debug(f"file_wordset_json")
    if key1 not in data.keys():
        data[key1] = {}
    logger.debug(f"file_wordset_json")
    if key2 not in data[key1].keys():
        data[key1][key2] = {}
    logger.debug(f"file_wordset_json")
    if key3 not in data[key1][key2].keys():
        data[key1][key2][key3] = {} 
    data[key1][key2][key3] = cluster_result_json
    logger.debug(f"file_wordset_json")
    with open(file_wordset_json, 'w', encoding='utf-8') as file_write:
        json.dump(data, file_write, ensure_ascii=False, indent=4)
    
        
def create_save_pic(km, X, img_file_path, df_cluster):
    logger.debug(f"开始画图")
    labels=km.labels_.tolist()
    label1 = [
    '#FFFF00', '#008008', '#0000FF','#800080','#FFF5EE','#98FB98','#A0522D',
    '#FF7F00','#FFC125','#FFFFFF','#FFFAFA','#FFF68F','#FFEFD5','#FFE4E1',
    '#FFDEAD','#FFC1C1','#FFB90F','#FFA54F','#FF8C00','#C0FF3E','#FF6EB4',
    '#FF4500','#FF3030','#8A2BE2','#87CEEB','#8470FF','#828282','#7EC0EE',
    '#7CFC00','#7A8B8B','#79CDCD','#76EE00']
    #l =  km.fit_predict(X)  
    # 2d图 df
    logger.debug(f"开始画2d图")
    n_component = 2
    svd = TruncatedSVD(n_components=n_component).fit(X)
    datapoint = svd.transform(X)
    plt.figure(figsize=(6, 6))
    colors = [label1[i] for i in labels]
    plt.scatter(datapoint[:, 0], datapoint[:, 1], c=colors)
    centroids = km.cluster_centers_
    centroidpoint = svd.transform(centroids)
    plt.scatter(centroidpoint[:, 0], centroidpoint[:, 1], marker='^', s=150, c='#000000')
    plt.title('SVD - 2 components')
    #plt.show()
    plt.savefig(img_file_path)
    logger.debug(f"2d图已保存")
    # 3d图
    logger.debug(f"开始画3d图")
    fig = plt.figure(figsize=(6, 6))
    ax = fig.add_subplot(111, projection='3d')
    # 为每个类别分配一种颜色
    # colors = ['r', 'g', 'b']
    colors = [label1[i] for i in labels]
    logger.debug(f"画3d图")
    svd = TruncatedSVD(n_components=3).fit(X)
    datapoint = svd.transform(X)
    cluster_labels = km.labels_
    reduced_vectors = datapoint
    # 3d图的x,y,z轴的label
    xyz_label = ['x axis','y axis','z axis']
    # 绘制每个类别的点
    """
    for label, color in enumerate(colors):
        points = reduced_vectors[cluster_labels == label]
        ax.scatter(points[:, 0], points[:, 1], points[:, 2], c=color)
        #ax.scatter(points[:, 0], points[:, 1], points[:, 2])
    """
    #points = reduced_vectors[cluster_labels == label]
    ax.scatter(datapoint[:, 0], datapoint[:, 1], datapoint[:, 2], c=colors)
    # 设置坐标轴标签和标题
    ax.set_xlabel(xyz_label[0])
    ax.set_ylabel(xyz_label[1])
    ax.set_zlabel(xyz_label[2])
    ax.set_title('SVD - 3 components(X)')
    # 显示图形
    # plt.show()  
    # 保存3d图
    plt.savefig(img_file_path.replace('2d','3d')) 
    # 三个方向画3d图
    # x方向
    plt.savefig(img_file_path.replace('2d','3d_x'))
    # y方向
    fig = plt.figure(figsize=(6, 6))
    ax = fig.add_subplot(111, projection='3d')
    ax.scatter(datapoint[:, 1], datapoint[:, 2], datapoint[:, 0], c=colors)
    ax.set_xlabel(xyz_label[1])
    ax.set_ylabel(xyz_label[2])
    ax.set_zlabel(xyz_label[0])
    ax.set_title('SVD - 3 components(Y)')
    plt.savefig(img_file_path.replace('2d','3d_y'))    
    # z方向
    fig = plt.figure(figsize=(6, 6))
    ax = fig.add_subplot(111, projection='3d')
    ax.scatter(datapoint[:, 2], datapoint[:, 0], datapoint[:, 1], c=colors)
    ax.set_xlabel(xyz_label[2])
    ax.set_ylabel(xyz_label[0])
    ax.set_zlabel(xyz_label[1])
    ax.set_title('SVD - 3 components(Z)')
    plt.savefig(img_file_path.replace('2d','3d_z'))
    # 饼图：250*250
    # 对列'A'进行分组，并对每个组计数
    grouped_count = df_cluster.groupby('类别')['类别'].count().reset_index(name='count')
    # 画饼图
    fig = plt.figure(figsize=(2.5, 2.5))
    plt.pie(grouped_count['count'], labels=grouped_count['类别'], autopct='%1.1f%%')
    plt.title('Class Proportion')  # 设置图表标题
    #plt.title('类别占比')  # 设置图表标题
    # plt.show()  # 显示图表
    plt.savefig(img_file_path.replace('2d','pie')) 
    logger.debug(f"画图完成")
    
    
# 聚类结果result.json样例
#CLUSTER_RESULT_JSON_SAMPLE = '{"modify": "","total": 0,"calc_date": "","cost": 0,"class-size": [],"Homogeneity": 0,"Completeness": 0,"V-measure": 0,"Adjusted-Rand-Index": 0,"Silhouette-Coefficient": 0}'
CLUSTER_RESULT_JSON_SAMPLE = '{"modify": "","total": 0,"feature-num": 0, "calc_date": "","cost": 0,"class-size": [],"Silhouette-Coefficient": 0, "Calinski-Harabasz-Index":0, "Davies-Bouldin-Index":0} '
# LDA结果result.json样例
LDA_RESULT_JSON_SAMPLE = """{"modify": "","total": 0,"feature-num": 0, "calc_date": "","cost": 0}"""
# 定义分词函数：输入字符串，返回以空格分隔的词串
def jieba_cut(text):
    return_text = ''
    if text:
        jieba_return = jieba.cut(text) #, cut_all=False))
        for word in jieba_return:
            if word not in STOPWORDS:
                return_text = return_text + ' ' + word
    return return_text

# 特征提取和聚类缺省参数
PARA_JSON_SAMPLE = {"TfidfVectorizer": {"max_df": 0.5,"min_df": 0.005,"stop_words": "english"},"TruncatedSVD": {"n_components": 500},"CountVectorizer": {"max_features": 500}}
# LDA缺省参数
PARA_JSON_SAMPLE_LDA ={"TfidfVectorizer": {"max_df": 0.5,"min_df": 0.005,"max_features": 100,"stop_words": "english"},"NMF": {"l1_ratio": 0.3}}    

# 读取聚类para json，没有则使用缺省参数
def get_para_dict(para_path, file_type, scale, category):
    # param_选词匹配表_feature_tfidf_class_5.json
    # C:/Users/work/GooSeekerTag/local/第31个任务/cluster
    # para文件名中的数据类型部分
    if file_type == 1:
        file_type_name = SOURCE[0]
    elif file_type == 2:
        file_type_name = SOURCE[1]
    elif file_type == 3:
        file_type_name = SOURCE[2] 
    # para文件名中的feature提取类型部分
    scale_name = ''
    if scale == 1:
        scale_name = '01'
    elif scale == 2:
        pass
    elif scale == 3:
        scale_name = 'tfidf'
    # para文件的路径
    para_file = os.path.join(para_path , f"param_{file_type_name}_feature_{scale_name}_class_{str(category)}.json")
    logger.debug(f"para_file: {para_file}")
    # 读取para文件
    para_file_json = {}
    if os.path.exists(para_file):
        try:
            with open(para_file, 'r', encoding='utf-8') as file_read:
                para_file_json = json.load(file_read)
                logger.debug(f"{para_file_json}")
        # 如果不是合法的json格式，则从DATA_SAMPLE文件load
        except json.JSONDecodeError:
            logger.error(f"{traceback.format_exc()}")
            logger.info(f"para_json文件格式非法，使用缺省参数")
    # 使用para文件读取的参数值替换缺省值
    para_json = PARA_JSON_SAMPLE
    # PARA_JSON_SAMPLE = {"TfidfVectorizer": {"max_df": 0.5,"min_df": 0.005,"stop_words": "english"},"TruncatedSVD": {"n_components": 500},"CountVectorizer": {"max_features": 500}}
    if para_file_json.get("TfidfVectorizer") and para_file_json.get("TfidfVectorizer").get("max_df"):
        para_json["TfidfVectorizer"]["max_df"] = para_file_json.get("TfidfVectorizer").get("max_df")
    if para_file_json.get("TfidfVectorizer") and para_file_json.get("TfidfVectorizer").get("min_df"):
        para_json["TfidfVectorizer"]["min_df"] = para_file_json.get("TfidfVectorizer").get("min_df")
    if para_file_json.get("TfidfVectorizer") and para_file_json.get("TfidfVectorizer").get("stop_words"):
        para_json["TfidfVectorizer"]["stop_words"] = para_file_json.get("TfidfVectorizer").get("stop_words")
    if para_file_json.get("TruncatedSVD") and para_file_json.get("TruncatedSVD").get("n_components"):
        para_json["TruncatedSVD"]["n_components"] = para_file_json.get("TruncatedSVD").get("n_components")
    if para_file_json.get("CountVectorizer") and para_file_json.get("CountVectorizer").get("max_features"):
        para_json["CountVectorizer"]["max_features"] = para_file_json.get("CountVectorizer").get("max_features")
    # 返回para_json
    logger.debug(f"{para_json}")
    return para_json

# 读取LDA para json，没有则使用缺省参数
def get_para_dict_lda(para_path, file_type, scale, topic, model):
    # param_选词匹配表_feature_tfidf_class_5.json
    # C:/Users/work/GooSeekerTag/local/第31个任务/cluster
    # $HOME/GooSeekerTag/local/马蜂窝游记分析/lda/param_原始数据表_feature_tfidf_topic_3_model_nmf.json
    # para文件名中的数据类型部分
    if file_type == 1:
        file_type_name = SOURCE[0]
    elif file_type == 2:
        file_type_name = SOURCE[1]
    elif file_type == 3:
        file_type_name = SOURCE[2] 
    # para文件名中的feature提取类型部分
    scale_name = ''
    if scale == 1:
        scale_name = '01'
    elif scale == 2:
        pass
    elif scale == 3:
        scale_name = 'tfidf'
    # para文件的路径
    para_file = os.path.join(para_path , f"param_{file_type_name}_feature_{scale_name}_topic_{str(topic)}_model_{model}.json")
    logger.debug(f"para_file: {para_file}")
    # 读取para文件
    para_file_json = {}
    if os.path.exists(para_file):
        try:
            with open(para_file, 'r', encoding='utf-8') as file_read:
                para_file_json = json.load(file_read)
                logger.debug(f"{para_file_json}")
        # 如果不是合法的json格式，则从DATA_SAMPLE文件load
        except json.JSONDecodeError:
            logger.error(f"{traceback.format_exc()}")
            logger.info(f"para_json文件格式非法，使用缺省参数")
    # 使用para文件读取的参数值替换缺省值
    para_json = PARA_JSON_SAMPLE_LDA
    # PARA_JSON_SAMPLE = {"TfidfVectorizer": {"max_df": 0.5,"min_df": 0.005,"max_features": 1000,"stop_words": "english"},"NMF": {"l1_ratio": 0.3}}
    if para_file_json.get("TfidfVectorizer") and para_file_json.get("TfidfVectorizer").get("max_df"):
        para_json["TfidfVectorizer"]["max_df"] = para_file_json.get("TfidfVectorizer").get("max_df")
    if para_file_json.get("TfidfVectorizer") and para_file_json.get("TfidfVectorizer").get("min_df"):
        para_json["TfidfVectorizer"]["min_df"] = para_file_json.get("TfidfVectorizer").get("min_df")
    if para_file_json.get("TfidfVectorizer") and para_file_json.get("TfidfVectorizer").get("max_features"):
        para_json["TfidfVectorizer"]["max_features"] = para_file_json.get("TfidfVectorizer").get("max_features")        
    if para_file_json.get("TfidfVectorizer") and para_file_json.get("TfidfVectorizer").get("stop_words"):
        para_json["TfidfVectorizer"]["stop_words"] = para_file_json.get("TfidfVectorizer").get("stop_words")
    if para_file_json.get("NMF") and para_file_json.get("NMF").get("l1_ratio"):
        para_json["NMF"]["l1_ratio"] = para_file_json.get("NMF").get("l1_ratio")
    # 返回para_json
    logger.debug(f"{para_json}")
    return para_json
    
def save_lda_result(lda_result_json, file_lda_json, file_type, scale, topic , model):
    logger.debug(f"file_lda_json文件：{file_lda_json}")
    data_json_sample = {}
    # 检查data.json是否存在
    if os.path.exists(file_lda_json):
        try:
            with open(file_lda_json, 'r', encoding='utf-8') as file_read:
                data = json.load(file_read)
        # 如果不是合法的json格式，则从DATA_SAMPLE文件load
        except json.JSONDecodeError:
            logger.error(f"{traceback.format_exc()}")
            logger.info(f"file_cluster_json文件：{file_lda_json} 格式非法，使用空的json")
            data = data_json_sample
    else:
        data = data_json_sample
    # 确定要更新的key
    key1 = SOURCE[file_type - 1]
    if scale == 3:
        key2 = f"feature_tfidf"
    else:
        key2 = f"feature_01"
    key3 = model
    key4 = f"topic_{topic}"
    # 检查并更新json
    logger.debug(f"file_lda_json文件：1")
    if key1 not in data.keys():
        data[key1] = {}
    logger.debug(f"file_lda_json文件：2")
    if key2 not in data[key1].keys():
        data[key1][key2] = {}
    logger.debug(f"file_lda_json文件：3")
    if key3 not in data[key1][key2].keys():
        data[key1][key2][key3] = {}
    if key4 not in data[key1][key2][key3].keys():
        data[key1][key2][key3][key4] = {}        
    data[key1][key2][key3][key4] = lda_result_json
    logger.debug(f"file_cluster_json文件：4")
    with open(file_lda_json, 'w', encoding='utf-8') as file_write:
        json.dump(data, file_write, ensure_ascii=False, indent=4)    
    
# 聚类  
def data_cluster(cluster_srcfilename, file_type, scale, category, file_data_json, file_cluster_json, para_dict, cluster_dir):
    logger.debug(f"cluster_srcfilename, file_type, scale, category：{cluster_srcfilename}, {file_type}, {scale}, {category}")
    calc_date = get_time()
    return_code = 0
    if os.path.exists(cluster_srcfilename):
        df_cluster_srcfilename = pd.read_excel(cluster_srcfilename)
        if file_type == 1:
            logger.debug("此处已执行1")
            df_cluster_srcfilename.dropna(subset=['正文'], inplace=True)
            df_cluster_srcfilename[COLUMN_CLUSTER]=df_cluster_srcfilename['正文'].apply(lambda x:jieba_cut(str(x)))
        elif file_type == 2:
            df_cluster_srcfilename.dropna(subset=['分词数据'], inplace=True)
            df_cluster_srcfilename[COLUMN_CLUSTER]=df_cluster_srcfilename['分词数据']
        elif file_type == 3:
            df_cluster_srcfilename.dropna(subset=['打标词'], inplace=True)
            df_cluster_srcfilename[COLUMN_CLUSTER]=df_cluster_srcfilename["原数据"]
            #df_cluster_srcfilename[COLUMN_CLUSTER]=df_cluster_srcfilename['打标词'].apply(lambda x:str(x).replace(',',' '))
        #df_cluster_srcfilename.to_excel('test.xlsx')
        # 删除字符串和NaN extension
        df_cluster_srcfilename[COLUMN_CLUSTER]=df_cluster_srcfilename[COLUMN_CLUSTER].replace('', np.nan)
        df_cluster_srcfilename.dropna(subset=[COLUMN_CLUSTER], inplace=True)
        logger.debug(df_cluster_srcfilename.head(1))
        #df_cluster_srcfilename_dropempty = df_cluster_srcfilename.replace('', np.nan)
        #df_cluster_srcfilename = df_cluster_srcfilename_dropempty.dropna(axis = 0, how ='any')
        #df_cluster_srcfilename = df_cluster_srcfilename.replace('', np.nan, inplace=True).dropna(axis = 0, how ='any', inplace=True)
        if not df_cluster_srcfilename.empty:
            t0 = time()
            logger.debug("此处已执行10")
            if scale == 1 or scale == 2:
                vectorizer, X = feature_extraction(file_type, df_cluster_srcfilename, scale, para_dict)
            elif scale == 3:
                vectorizer, X, lsa_0 = feature_extraction(file_type, df_cluster_srcfilename, scale, para_dict)    
            logger.info("完成所耗费时间： %fs" % (time() - t0))
            logger.info("样本数量: %d, 特征数量: %d" % X.shape)
            logger.info('特征抽取完成！')
            #labels = df_cluster_srcfilename[COLUMN_CLUSTER]
            n_init_number=5
            init_method='k-means++'
            cluster_result_json = json.loads(CLUSTER_RESULT_JSON_SAMPLE)
            km = KMeans(n_clusters=category, init=init_method, max_iter=300, n_init=n_init_number)
            # cluster_result_json["feature-num"] = para_dict["TruncatedSVD"]["n_components"]
            cluster_result_json["feature-num"] = X.shape[1]
            #n_compo = 100
            km.fit(X)
            class_size = []
            for i in range(category):
                class_size.append(int(list(km.predict(X)).count(i)))
            # 计算
            Silhouette_Coefficient = metrics.silhouette_score(X, km.labels_, sample_size=1000)
            logger.debug(f"X TYPE: {type(X)}")
            # X是scipy.sparse._csr.csr_matrix类型，需要转换成array：X.toarray()
            if str(type(X)) == "<class 'scipy.sparse._csr.csr_matrix'>":
                Calinski_Harabasz_Index = metrics.calinski_harabasz_score(X.toarray(), km.labels_)
                logger.debug(f"Calinski_Harabasz_Index：{Calinski_Harabasz_Index}")
                Davies_Bouldin_Index = metrics.davies_bouldin_score(X.toarray(), km.labels_)
                logger.debug(f"Davies_Bouldin_Index{Davies_Bouldin_Index}")
            else:
                Calinski_Harabasz_Index = metrics.calinski_harabasz_score(X, km.labels_)
                logger.debug(f"Calinski_Harabasz_Index：{Calinski_Harabasz_Index}")
                Davies_Bouldin_Index = metrics.davies_bouldin_score(X, km.labels_)
                logger.debug(f"Davies_Bouldin_Index{Davies_Bouldin_Index}")       
            # 获取类别中的特征词
            if scale == 3:
                order_centroids = lsa_0.inverse_transform(km.cluster_centers_).argsort()[:, ::-1]
            else:
                order_centroids = km.cluster_centers_.argsort()[:, ::-1]
            #original_space_centroids = lsa_0.inverse_transform(km.cluster_centers_)
            #order_centroids = original_space_centroids.argsort()[:, ::-1]
            # 由于sklearn存在不同版本，这里检测后再调用get_feature_names_out或者get_feature_names
            if hasattr(vectorizer,'get_feature_names_out'):
                terms = vectorizer.get_feature_names_out()
            elif hasattr(vectorizer,'get_feature_names'):
                terms = vectorizer.get_feature_names()
            wordset_json = []
            for i in range(category):
                wordset = ""
                for ind in order_centroids[i, :]:
                    if terms[ind].replace(' ','') != ',' and len(terms[ind].replace(' ',''))>0:
                        wordset = ','.join([wordset, terms[ind].replace(' ','')])
                wordset_json.append(wordset.strip(','))        
            # 聚类时长
            cost = time() - t0
            # 从保存的json文件里读取数据的修改时间
            modify = get_value_from_data_json(file_data_json, file_type, 'modify')
            # 更新cluster_result_json
            cluster_result_json["Calinski-Harabasz-Index"] = Calinski_Harabasz_Index 
            cluster_result_json["Davies-Bouldin-Index"] = Davies_Bouldin_Index
            cluster_result_json["modify"] = modify
            cluster_result_json["total"] = len(df_cluster_srcfilename)
            cluster_result_json["calc_date"] = calc_date
            cluster_result_json["cost"] = cost
            cluster_result_json["class-size"] = class_size
            cluster_result_json["Silhouette-Coefficient"] = Silhouette_Coefficient
            logger.debug(f"cluster_result_json：{cluster_result_json}")
            # 写入cluster_result.json文件
            save_cluster_result(cluster_result_json, file_cluster_json, file_type, scale, category)
            # dataframe增加“类别”这1列
            df_cluster_srcfilename_temp = df_cluster_srcfilename.copy()
            df_cluster_srcfilename_temp["类别"]=km.labels_
            df_cluster_srcfilename=df_cluster_srcfilename_temp.drop([COLUMN_CLUSTER],axis=1)
            # dataframe写入cluster.xlsx和csv文件   
            # C:\Users\work\GooSeekerTag\local\第40个任务\cluster\raw\原始数据表.xlsx
            cluster_dir_processed = os.path.join(cluster_dir , 'processed')
            cluster_file_xlsx = os.path.basename(cluster_srcfilename).split('.')[0] + '_' + FEATURE[scale - 1] + '_class_' + str(category) + '_cluster' + '.xlsx'
            cluster_file_csv = os.path.basename(cluster_srcfilename).split('.')[0] + '_' + FEATURE[scale - 1] + '_class_' + str(category) + '_cluster' + '.csv'
            cluster_file_xlsx_full = os.path.join(cluster_dir_processed ,cluster_file_xlsx)
            cluster_file_csv_full = os.path.join(cluster_dir_processed ,cluster_file_csv)
            logger.debug(f"cluster_dir + '/' + cluster_file_xlsx：{cluster_file_xlsx_full}")
            df_cluster_srcfilename.to_excel(cluster_file_xlsx_full , index=False)
            logger.debug(f"cluster_dir + '/' + cluster_file_csv：{cluster_file_csv_full}")
            df_cluster_srcfilename.to_csv(cluster_file_csv_full , index=False)
            # wordset_json数据生成
            file_wordset_json = cluster_file_xlsx_full.replace('cluster.xlsx','wordset.json')
            #logger.debug(wordset_json)
            #logger.debug(len(wordset_json))
            # 写入wordset_json文件
            save_wordset_result(wordset_json, file_wordset_json, file_type, scale, category)
            # 保存2d/3d/pie图
            file_image_2d = os.path.join(cluster_dir_processed.replace('processed','img') , cluster_file_xlsx.replace('cluster.xlsx','2d.png'))
            logger.debug(f"2d/3d/pie picture will be saved under：{cluster_dir.replace('processed','img')}")
            logger.debug(f"{X.shape[0]}")
            logger.debug(f"{X.shape[1]}")
            create_save_pic(km, X, file_image_2d, df_cluster_srcfilename)
            return_code = 0
        else:
            logger.info(f"无有效的聚类源数据")
            return_code = 106
    else:
        logger.info(f"cluster_srcfilename not exist：{cluster_srcfilename}")
        return_code = 105
    return return_code
    
# ###############################################
# lda相关
# ###############################################
# LDA  
INIT = "nndsvda"

# 保存单个子图
def save_subfig(fig,ax,picfilename):
    bbox = ax.get_tightbbox(fig.canvas.get_renderer()).expanded(1.02,1.02)
    extent = bbox.transformed(fig.dpi_scale_trans.inverted())
    fig.savefig(picfilename, bbox_inches=extent)

# LDA画图函数
def plot_top_words(model, tf, tf_vectorizer, feature_names, n_top_words, title, rows, columns, topicnum, scale_name, model_name, lda_dir, data_name ,dfsource):
    return_code = 0
    try:
        # 设置中文字体
        current_os = get_os()
        logger.debug(f"operating system: {current_os}")
        if current_os == 'windows':
            plt.rcParams['font.sans-serif'] = ['SimHei']
        else:
            plt.rcParams['font.sans-serif'] = ['Arial Unicode MS']
            plt.rcParams['axes.unicode_minus']=False
        fig, axes = plt.subplots(rows, columns, figsize=(10, 8*rows), sharex=True)
        axes = axes.flatten()
        logger.info(f"axes length: {len(axes)}")
        for topic_idx, topic in enumerate(model.components_):
            top_features_ind = topic.argsort()[-n_top_words:]
            top_features = feature_names[top_features_ind]
            weights = topic[top_features_ind]

            ax = axes[topic_idx]
            ax.barh(top_features, weights, height=0.7)
            ax.set_title(f"Topic {topic_idx +1}", fontdict={"fontsize": 10})
            ax.tick_params(axis="both", which="major", labelsize=10)
            #fig.savefig(f"Topic_{topic_idx +1}.png")
            for i in "top right left".split():
                ax.spines[i].set_visible(False)
            fig.suptitle(title, fontsize=20)
        plt.subplots_adjust(top=0.90, bottom=0.05, wspace=0.90, hspace=0.3)
        picname = f"{data_name}_feature_{scale_name}_topic_{topicnum}_{model_name}.png"
        plt.savefig(os.path.join(lda_dir, 'img', picname))
        for model_num in range(min(len(axes),topicnum)):
            picname = f"{data_name}_feature_{scale_name}_topic_{topicnum}_{model_num+1}_{model_name}.png"
            save_subfig(fig,axes[model_num],os.path.join(lda_dir,'img', picname ))
        # plt.show()
        try:
            # 使用pyLDAvis保存可视化html
            logger.debug(f"model type{type(model)}")
            logger.debug(f"{model}")
            logger.debug(f"tf type{type(tf)}")
            logger.debug(f"{tf}")
            logger.debug(f"tf_vectorizer type{type(tf_vectorizer)}")
            logger.debug(f"{tf_vectorizer}")
            # 解决pyinstaller打包后运行时出错的问题
            from joblib import parallel_backend
            with parallel_backend('threading', n_jobs=1):
                prepared_data = pyLDAvis.lda_model.prepare(model, tf, tf_vectorizer, n_jobs=1)
                logger.debug(f"prepared_data type{type(prepared_data)}")
                htmlname = f"{data_name}_feature_{scale_name}_topic_{topicnum}_{model_name}_vis.html"
                pyLDAvis.save_html(prepared_data, os.path.join(lda_dir, 'processed', htmlname))
                logger.info(f"pyLDAvis {model_name} 保存html成功")
        except Exception as e:
            #logger.error('异常信息:' + str(e))
            logger.error(f"{traceback.format_exc()}")
            logger.info(f"pyLDAvis {model_name} 保存html失败")
    except Exception as e:
        #logger.error('异常信息:' + str(e))
        logger.error(f"{traceback.format_exc()}")
        return_code = 108
    return return_code

def data_lda(lda_srcfilename_xlsx, file_type, scale, topic , model , task, file_data_json_lda, file_lda_json, para_dict, lda_dir):
    calc_date = get_time()
    return_code = 0
    if os.path.exists(lda_srcfilename_xlsx):
        df_lda_srcfilename = pd.read_excel(lda_srcfilename_xlsx)
        logger.debug(f"df_lda_srcfilename shape：{df_lda_srcfilename.shape}")
        if file_type == 1:
            data_name = '原始数据表'
            logger.debug("此处已执行1")
            df_lda_srcfilename.dropna(subset=['正文'], inplace=True)
            df_lda_srcfilename[COLUMN_CLUSTER]=df_lda_srcfilename['正文'].apply(lambda x:jieba_cut(str(x)))
        elif file_type == 2:
            data_name = '分词效果表'
            df_lda_srcfilename.dropna(subset=['分词数据'], inplace=True)
            df_lda_srcfilename[COLUMN_CLUSTER]=df_lda_srcfilename['分词数据']
        elif file_type == 3:
            data_name = '选词匹配表'
            df_lda_srcfilename.dropna(subset=['打标词'], inplace=True)
            df_lda_srcfilename[COLUMN_CLUSTER]=df_lda_srcfilename["打标词"].apply(lambda x:str(x).replace(',',' '))
        # 删除字符串和NaN extension
        df_lda_srcfilename[COLUMN_CLUSTER]=df_lda_srcfilename[COLUMN_CLUSTER].replace('', np.nan)
        df_lda_srcfilename.dropna(subset=[COLUMN_CLUSTER], inplace=True)
        # 原数据表增加1列数字序列：1,2......
        df_lda_srcfilename['origin_index'] = range(1, len(df_lda_srcfilename) + 1)
        logger.debug(f"df_lda_srcfilename shape：{df_lda_srcfilename.shape}")
        # 由于原数据表drop了一些行，index不是连续的，和lda/nmf使用的df_lda_srcfilename[COLUMN_CLUSTER].values.astype('U')自动生成的从0开始的index不对应
        # 所以此处为原数据表生成新的从0开始的index
        df_lda_srcfilename.reset_index(drop=True, inplace=True)
        if not df_lda_srcfilename.empty:
            t0 = time()
            logger.debug("此处已执行10")
            # {"TfidfVectorizer": {"max_df": 0.5,"min_df": 0.005,"max_features": 100,"stop_words": "english"},"NMF": {"l1_ratio": 0.3}}
            if scale == 1 or scale == 2:
                scale_name = '01'
                logger.debug("Use countvectorizer to extract features...")
                tf_vectorizer = CountVectorizer(max_df=para_dict["TfidfVectorizer"]["max_df"], min_df=para_dict["TfidfVectorizer"]["min_df"], max_features=para_dict["TfidfVectorizer"]["max_features"], stop_words=para_dict["TfidfVectorizer"]["stop_words"])
                tf = tf_vectorizer.fit_transform(df_lda_srcfilename[COLUMN_CLUSTER].values.astype('U'))
                # filter out terms
                ids = np.array(tf.sum(axis=1)==0).ravel()
                tfidf_filtered = tf[~ids]
                tf = tfidf_filtered
                logger.info("完成特征提取所耗费时间： %fs" % (time() - t0))
                logger.info("样本数量: %d, 特征数量: %d" % tf.shape)
                logger.info('特征抽取完成！')
            elif scale == 3:
                scale_name = 'tfidf'
                logger.debug("Use TfidfVectorizer to extract features...")
                tf_vectorizer = TfidfVectorizer(max_df=para_dict["TfidfVectorizer"]["max_df"], min_df=para_dict["TfidfVectorizer"]["min_df"], max_features=para_dict["TfidfVectorizer"]["max_features"], stop_words=para_dict["TfidfVectorizer"]["stop_words"])
                tf = tf_vectorizer.fit_transform(df_lda_srcfilename[COLUMN_CLUSTER].values.astype('U'))
                # filter out terms
                ids = np.array(tf.sum(axis=1)==0).ravel()
                tfidf_filtered = tf[~ids]
                tf = tfidf_filtered
                logger.info("完成特征提取所耗费时间： %fs" % (time() - t0))
                logger.info("样本数量: %d, 特征数量: %d" % tf.shape)
                logger.info('特征抽取完成！')
            if model == 'nmf' or model == 'lda':
                if model == 'nmf':
                    logger.debug("Fitting the NMF model (Frobenius norm) with tf-idf features, ")
                    logger.debug(f"n_samples={tf.shape[0]} and n_features={tf.shape[1]}.")
                    t0 = time()
                    nmf = NMF(
                        n_components=topic,
                        random_state=1,
                        init=INIT,
                        beta_loss="frobenius",
                        alpha_W=0.00005,
                        alpha_H=0.00005,
                        l1_ratio=para_dict["NMF"]["l1_ratio"],
                        max_iter=10000,
                        tol=1e-6
                    )
                    # tf = tf.inverse_transform(X)
                    nmf_result = nmf.fit(tf)
                    logger.info("nmf calculate done in %0.3fs." % (time() - t0))
                    # 前20个词
                    n_top_words = 20
                    feature_names = tf_vectorizer.get_feature_names_out()
                    # $HOME/GooSeekerTag/local/马蜂窝游记分析/lda/img/原始数据表_feature_tfidf_topic_3_lda.png 
                    # 画图并保存png和html
                    result_model = nmf_result
                    title = "Topics in NMF model"
                elif model == 'lda':
                    logger.debug("Fitting the LDA model with tf-idf features, ")
                    logger.debug(f"n_samples={tf.shape[0]} and n_features={tf.shape[1]}.")
                    t0 = time()
                    lda = LatentDirichletAllocation(
                        n_components=topic,
                        max_iter=50,
                        # learning_method="online",
                        learning_method="batch",
                        learning_offset=50.0,
                        random_state=0,
                    )
                    lda_result = lda.fit(tf)
                    logger.info("lda calculate done in %0.3fs." % (time() - t0))
                    # 前20个词
                    n_top_words = 20
                    feature_names = tf_vectorizer.get_feature_names_out()
                    result_model = lda_result
                    title = "Topics in LDA model"
                # LDA时长
                cost = time() - t0  
                logger.debug(f"cost：{cost}")
                calc_date = get_time()
                logger.debug(f"calc_date：{calc_date}")
                # 从保存的json文件里读取数据的修改时间
                modify = get_value_from_data_json(file_data_json_lda, file_type, 'modify')  
                logger.debug(f"modify：{modify}")
                # 读取缺省值
                lda_result_json = json.loads(LDA_RESULT_JSON_SAMPLE)
                # save json
                lda_result_json["modify"] = modify
                lda_result_json["feature-num"] = tf.shape[1]
                lda_result_json["cost"] = cost
                lda_result_json["calc_date"] = calc_date
                lda_result_json["total"] = tf.shape[0]
                # 写入lda_result.json文件
                logger.debug(f"{lda_result_json}, {file_lda_json}, {file_type}, {scale}, {topic} , {model}")
                save_lda_result(lda_result_json, file_lda_json, file_type, scale, topic , model)
                # 特征词-话题
                column_names = ['特征词']
                for i in range(topic):
                    column_names.append(f"话题{i+1}")
                logger.debug(f"{column_names}")
                df_topic_features = pd.DataFrame(columns=column_names)
                tfidf_feature_names = tf_vectorizer.get_feature_names_out()
                logger.debug(f"{tfidf_feature_names.shape}")
                H = result_model.components_
                logger.debug(f"{H.shape}")
                # 新增一行数据
                for topic_idx, topic_H in enumerate(H):
                    #logger.debug(f"topic_idx: {topic_idx}")
                    #logger.debug(f"topic_H: {topic_H}")
                    # top_features_ind = topic_H.argsort()[-n_top_words:]
                    top_features_ind = topic_H.argsort()[:]
                    top_features = tfidf_feature_names[top_features_ind]
                    weights = topic_H[top_features_ind]
                    for feature_index in range(len(top_features)):
                        row_index_search = df_topic_features['特征词'].index[df_topic_features['特征词'] == str(top_features[feature_index])].tolist()
                        if row_index_search:
                            df_topic_features.at[row_index_search[0],f"话题{topic_idx + 1}"]=weights[feature_index]
                        else:
                            row_index = len(df_topic_features)
                            df_topic_features.at[row_index,f"特征词"]= top_features[feature_index]
                            df_topic_features.at[row_index,f"话题{topic_idx + 1}"]= weights[feature_index]
                # 保存特征词-话题
                df_topic_features.to_excel(os.path.join(lda_dir, 'processed', f"{data_name}_feature_{scale_name}_topic_{topic}_{model}_word_trans.xlsx"), index=False)
                df_topic_features.to_csv(os.path.join(lda_dir, 'processed', f"{data_name}_feature_{scale_name}_topic_{topic}_{model}_word_trans.csv"), index=False)
                logger.info(f"{model}特征词-话题表已保存")
                # 数据-话题
                column_names = ['序号','origin_index','原数据','打标词']
                for i in range(topic):
                    column_names.append(f"话题{i+1}")
                # 生成空的原文-话题表
                df_topic_data = pd.DataFrame(columns=column_names)
                if model == 'lda':
                    W = lda.transform(tf)
                else:
                    W = nmf.fit_transform(tf)
                logger.debug(f"W shape: {W.shape}")
                WT = W.transpose()
                for topic_idx, topic_WT in enumerate(WT):
                    # logger.debug(f"topic_idx: {topic_idx}")
                    # 每个话题选择输出30个原文
                    # n_top_data = 30
                    # top_data_ind = topic_WT.argsort()[-n_top_data:]
                    top_data_ind = topic_WT.argsort()[:]
                    weights = topic_WT[top_data_ind]
                    # logger.debug(f"{type(top_data_ind)}")
                    # logger.debug(f"{type(weights)}")
                    # logger.debug(f"{top_data_ind}")
                    # logger.debug(f"{weights}")
                    # logger.debug(f"len(top_data_ind):{len(top_data_ind)}")
                    for top_data_ind_idx in range(len(top_data_ind)):
                        content = str(df_lda_srcfilename['正文'][top_data_ind[top_data_ind_idx]]) if file_type == 1 else str(df_lda_srcfilename['原数据'][top_data_ind[top_data_ind_idx]])
                        xuhao = str(df_lda_srcfilename['序号'][top_data_ind[top_data_ind_idx]])
                        if file_type == 3:
                            dabiaoci = str(df_lda_srcfilename['打标词'][top_data_ind[top_data_ind_idx]])
                        origin_index_number = df_lda_srcfilename['origin_index'][top_data_ind[top_data_ind_idx]]
                        #row_index_search = df_topic_data['原数据'].index[(df_topic_data['原数据'] == content) and (df_topic_data['序号'] == xuhao)].tolist()
                        row_index_search = df_topic_data[(df_topic_data['原数据'] == content) & (df_topic_data['序号'] == xuhao)].index.tolist()
                        if row_index_search:
                            df_topic_data.at[row_index_search[0],f"话题{topic_idx + 1}"]=weights[top_data_ind_idx]
                        else:
                            row_index = len(df_topic_data)
                            df_topic_data.at[row_index,f"原数据"]= content
                            df_topic_data.at[row_index,f"序号"]= xuhao 
                            if file_type == 3:
                                df_topic_data.at[row_index,f"打标词"]= dabiaoci
                            df_topic_data.at[row_index,f"origin_index"]= origin_index_number
                            df_topic_data.at[row_index,f"话题{topic_idx + 1}"]= weights[top_data_ind_idx]
                if file_type != 3:
                    df_topic_data.rename(columns={'原数据': '正文'}, inplace=True)
                    df_topic_data = df_topic_data.drop('打标词', axis=1)
                # 保存原数据-话题
                df_topic_data_sorted = df_topic_data.sort_values(by='origin_index', ascending=True)
                df_topic_data_sorted = df_topic_data_sorted.drop('origin_index', axis=1)
                df_topic_data_sorted.to_excel(os.path.join(lda_dir, 'processed', f"{data_name}_feature_{scale_name}_topic_{topic}_{model}.xlsx"), index=False)
                df_topic_data_sorted.to_csv(os.path.join(lda_dir, 'processed', f"{data_name}_feature_{scale_name}_topic_{topic}_{model}.csv"), index=False)
                logger.info(f"{model}原数据-话题表已保存")
                # 画图并保存png和html
                rows = math.ceil(topic / 5)
                columns = 5
                if model == 'nmf':
                    return_code = plot_top_words(result_model, tf, tf_vectorizer, feature_names, n_top_words, title,rows, columns,topic, scale_name, model ,lda_dir, data_name,df_lda_srcfilename)
                else:
                    return_code = plot_top_words(result_model, tf, tf_vectorizer, feature_names, n_top_words, title,rows, columns,topic, scale_name, model ,lda_dir, data_name,df_lda_srcfilename)
            else:
                return_code = 107
        else:
            logger.info(f"无有效的聚类源数据")
            return_code = 106
    else:
        logger.info(f"lda_srcfilename_xlsx not exist：{lda_srcfilename_xlsx}")
        return_code = 105
    return return_code
    
# Press the green button in the gutter to run the script.
if __name__ == '__main__':
    # 定义命令行参数
    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--version', action='version', version=get_version(), help='显示当前版本号')
    parser.add_argument("-i", "--input", help="包括完整目录的数据文件名，在导入数据时此参数必须提供。", required=False)
    parser.add_argument("-f", "--file_type", type=int, choices=[0, 1, 2, 3, 4], default=0, help="在导入数据时此参数必须提供。参数的值定义：0.保留(input文件如果是excel格式将会以csv格式保存到raw目录)； 1.原始数据表；2.分词效果表；3.选词匹配表；4.自动根据文件类型和字段名称导入原始数据表/分词效果表/选词匹配表。", required=False)
    parser.add_argument("-t", "--task", help="任务名称，此参数必须提供。", required=True)
    parser.add_argument("-m", "--home", help="数据存放目录，此参数是可选参数，如果不提供，则缺省目录是: $HOME/GooSeekerTag/local")
    parser.add_argument("-s", "--scale", type=int, choices=[1, 2, 3], help="在聚类时此参数必须提供。特征词赋值类型, 类型1: 0/1， 类型2: 词频，类型3: tf-idf。", required=False)
    parser.add_argument("-c", "--category", type=int, help="在聚类时此参数必须提供, 类别数。", required=False)
    parser.add_argument("-p", "--topic", type=int, default=0, help="lda必须提供此参数, 代表话题数。", required=False)
    parser.add_argument("-d", "--model", choices=["nmf", "lda"], help='lda计算模型，取值["nmf", "lda"]', required=False)
    parser.add_argument("-o", "--operate", choices=['lda'], help="表示进行聚类数据导入。", required=False)
    parser.add_argument("--loglevel", help="可选参数：日志级别", choices=['TRACE', 'DEBUG', 'INFO', 'SUCCESS', 'WARNING', 'ERROR', 'CRITICAL'])
    args = parser.parse_args()
    # print(args)
    # 当前用户主目录
    user_home_dir = unix_path(os.path.expanduser('~'))
    # 数据存放目录缺省为$HOME/GooSeekerTag/local
    gooseeker_tag_local = os.path.join(user_home_dir , DEFAULT_CLUSTER_DIR)
    # 判断命令行是否有-m参数
    if args.home:
        gooseeker_tag_local = unix_path(args.home)
        DIR_LOG = os.path.join(gooseeker_tag_local,'..', 'log')
        FILE_LOG = os.path.join(DIR_LOG,'cluster.log')
    # tag任务目录初始化
    if args.operate or args.topic:
        tag_dir_init(gooseeker_tag_local, args.task, 'lda')
    else:
        tag_dir_init(gooseeker_tag_local, args.task)
    # 启动日志
    # 命令行是否指定了日志级别
    if args.loglevel:
        LOG_LEVEL = args.loglevel
    logger.remove()
    logger.add(sys.stdout, filter=lambda record: record["level"].no < 40, level=LOG_LEVEL, format="{message}")
    handler_id = logger.add(sys.stderr, level='ERROR', format="{message}")#添加一个可以修改控制的handler
    #file_id = logger.add(os.path.join(gooseeker_tag_local ,args.task , FILE_LOG), level=LOG_LEVEL, rotation="1GB", encoding="utf8")
    file_id = logger.add(FILE_LOG, level=LOG_LEVEL, rotation="1GB", encoding="utf8")
    logger.debug(f"start .....")

    # 命令行参数写入日志
    logger.debug(f"{args}")
    # logger.info(f"命令行参数 args.input: {args.input}")
    # logger.info(f"命令行参数 args.file_type: {args.file_type}")
    # logger.info(f"命令行参数 args.task: {args.task}")
    # logger.info(f"命令行参数 args.home: {args.home}")
    # logger.info(f"命令行参数 args.scale: {args.scale}")
    # logger.info(f"命令行参数 args.category: {args.category}")
    logger.debug(f"cluster dir: {gooseeker_tag_local}")
    
    try:
        # 测试
        # open('file_not_exist.xlsx', 'r', encoding='utf-8')
        # 导入数据
        if args.input and args.file_type and args.task:
            logger.info(f"开始导入数据")
            inputfile = unix_path(args.input)
            logger.debug("inputfile_extension = inputfile.split('.')[-1]")
            inputfile_extension = inputfile.split('.')[-1]
            # 检查import文件是否存在
            if not os.path.exists(inputfile):
                logger.error(f"导入文件: {inputfile}不存在")
                exit_code = 101
            # file_type值：0,1,2,3,4
            elif not inputfile_extension in FILE_EXTENSION_ALL:
                logger.error(f"导入文件后缀:{inputfile_extension}不支持")
                exit_code = 102
            elif args.file_type == 0:
                pass
            elif args.file_type == 1:
                if inputfile_extension in FILE_EXTENSION_ORIGIN_DATA:
                    if args.operate:
                        file_origin_data_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_ORIGIN_DATA_XLSX)
                        file_origin_data_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_ORIGIN_DATA_CSV)
                        file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON_LDA)
                    else:
                        file_origin_data_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_ORIGIN_DATA_XLSX)
                        file_origin_data_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_ORIGIN_DATA_CSV)
                        file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON)                        
                    # 导入-i或--input指定的原始数据文件
                    modify = get_time()
                    total = import_file_origin_data(file_origin_data_xlsx, args.file_type, inputfile)
                    logger.debug(f"总记录数: {total}.  导入时间: {modify}")
                    if total > 0:
                        logger.debug(f"{file_origin_data_xlsx} created/updated")
                        logger.debug(f"{file_origin_data_csv} created/updated")
                        # 更新data_json
                        update_json(file_data_json, args.file_type, modify, total)
                        logger.debug(f"{file_data_json} updated")
                else:
                    logger.error(f"导入文件后缀:{inputfile_extension}不支持")
                    exit_code = 102
            elif args.file_type == 2:
                if inputfile_extension in FILE_EXTENSION_SEG_EFFECT:
                    if args.operate:
                        file_seg_effect_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_SEG_EFFECT_XLSX)
                        file_seg_effect_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_SEG_EFFECT_CSV)
                        file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON_LDA)
                    else:
                        file_seg_effect_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_SEG_EFFECT_XLSX)
                        file_seg_effect_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_SEG_EFFECT_CSV)
                        file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON)
                    # 导入-i或--input指定的原始数据文件
                    modify = get_time()
                    total = import_file_seg_effect(file_seg_effect_xlsx, args.file_type, inputfile)
                    logger.debug(f"总记录数: {total}.  导入时间: {modify}")
                    if total > 0:
                        logger.debug(f"{file_seg_effect_xlsx} created/updated")
                        logger.debug(f"{file_seg_effect_csv} created/updated")
                        # 更新data_json
                        update_json(file_data_json, args.file_type, modify, total)
                        logger.debug(f"{file_data_json} updated")
                else:
                    logger.error(f"导入文件后缀:{inputfile_extension}不支持")
                    exit_code = 102
            elif args.file_type == 3:
                if inputfile_extension in FILE_EXTENSION_WORD_CHOICE_MATCH:
                    if args.operate:
                        file_word_choice_match_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_WORD_CHOICE_MATCH_XLSX)
                        file_word_choice_match_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_WORD_CHOICE_MATCH_CSV)
                        file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON_LDA)
                    else:
                        file_word_choice_match_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_WORD_CHOICE_MATCH_XLSX)
                        file_word_choice_match_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_WORD_CHOICE_MATCH_CSV)
                        file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON)
                    # 导入-i或--input指定的原始数据文件
                    modify = get_time()
                    total = import_file_word_choice_match(file_word_choice_match_xlsx, args.file_type, inputfile)
                    logger.debug(f"总记录数: {total}.  导入时间: {modify}")
                    if total > 0:
                        logger.debug(f"{file_word_choice_match_xlsx} created/updated")
                        logger.debug(f"{file_word_choice_match_csv} created/updated")
                        # 更新data_json
                        update_json(file_data_json, args.file_type, modify, total)
                        logger.debug(f"{file_data_json} updated")
                else:
                    logger.error(f"导入文件后缀:{inputfile_extension}不支持")
                    exit_code = 102
            elif args.file_type == 4:
                if inputfile_extension in ['docx','pdf','txt','csv','xls']:
                    logger.info(f"{inputfile}将会被导入到{FILE_ORIGIN_DATA_XLSX}")
                    if args.operate:
                        file_origin_data_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_ORIGIN_DATA_XLSX)
                        file_origin_data_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_ORIGIN_DATA_CSV)
                        file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON_LDA)
                    else:
                        file_origin_data_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_ORIGIN_DATA_XLSX)
                        file_origin_data_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_ORIGIN_DATA_CSV)
                        file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON)
                    # 导入-i或--input指定的原始数据文件
                    modify = get_time()
                    total = import_file_origin_data(file_origin_data_xlsx, 1, inputfile)
                    logger.debug(f"总记录数: {total}.  导入时间: {modify}")
                    if total > 0:
                        logger.debug(f"{file_origin_data_xlsx} created/updated")
                        logger.debug(f"{file_origin_data_csv} created/updated")
                        # 更新data_json
                        update_json(file_data_json, 1, modify, total)
                        logger.debug(f"{file_data_json} updated")
                    exit_code = 1
                else:
                    # 检查excel文件的表头
                    # 如果有："序号，正文"字段，则为原始数据表
                    # 如果有："序号，分词数据"字段，则为分词效果表
                    # 如果有："序号，打标词"字段，则为选词匹配表
                    data_type = check_data_type(inputfile)
                    # 
                    if data_type == 1:
                        logger.info(f"{inputfile}将会被导入到{FILE_ORIGIN_DATA_XLSX}")
                        if args.operate:
                            file_origin_data_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_ORIGIN_DATA_XLSX)
                            file_origin_data_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_ORIGIN_DATA_CSV)
                            file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON_LDA)
                        else:
                            file_origin_data_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_ORIGIN_DATA_XLSX)
                            file_origin_data_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_ORIGIN_DATA_CSV)
                            file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON)
                        # 导入-i或--input指定的原始数据文件
                        modify = get_time()
                        total = import_file_origin_data(file_origin_data_xlsx, data_type, inputfile)
                        logger.debug(f"总记录数: {total}.  导入时间: {modify}")
                        if total > 0:
                            logger.debug(f"{file_origin_data_xlsx} created/updated")
                            logger.debug(f"{file_origin_data_csv} created/updated")
                            # 更新data_json
                            update_json(file_data_json, data_type, modify, total)
                            logger.debug(f"{file_data_json} updated")
                        exit_code = 1
                    elif data_type == 2:
                        logger.info(f"{inputfile}将会被导入到{FILE_SEG_EFFECT_XLSX}")
                        if args.operate:
                            file_seg_effect_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_SEG_EFFECT_XLSX)
                            file_seg_effect_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_SEG_EFFECT_CSV)
                            file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON_LDA)
                        else:
                            file_seg_effect_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_SEG_EFFECT_XLSX)
                            file_seg_effect_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_SEG_EFFECT_CSV)
                            file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON)
                        # 导入-i或--input指定的原始数据文件
                        modify = get_time()
                        total = import_file_seg_effect(file_seg_effect_xlsx, data_type, inputfile)
                        logger.debug(f"总记录数: {total}.  导入时间: {modify}")
                        if total > 0:
                            logger.debug(f"{file_seg_effect_xlsx} created/updated")
                            logger.debug(f"{file_seg_effect_csv} created/updated")
                            # 更新data_json
                            update_json(file_data_json, data_type, modify, total)
                            logger.debug(f"{file_data_json} updated")
                        exit_code = 2
                    elif data_type == 3:
                        logger.info(f"{inputfile}将会被导入到{FILE_WORD_CHOICE_MATCH_XLSX}")
                        if args.operate:
                            file_word_choice_match_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_WORD_CHOICE_MATCH_XLSX)
                            file_word_choice_match_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_WORD_CHOICE_MATCH_CSV)
                            file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON_LDA)
                        else:
                            file_word_choice_match_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_WORD_CHOICE_MATCH_XLSX)
                            file_word_choice_match_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_WORD_CHOICE_MATCH_CSV)
                            file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON)
                        # 导入-i或--input指定的原始数据文件
                        modify = get_time()
                        total = import_file_word_choice_match(file_word_choice_match_xlsx, data_type, inputfile)
                        logger.debug(f"总记录数: {total}.  导入时间: {modify}")
                        if total > 0:
                            logger.debug(f"{file_word_choice_match_xlsx} created/updated")
                            logger.debug(f"{file_word_choice_match_csv} created/updated")
                            # 更新data_json
                            logger.debug(f"before update json")
                            logger.debug(f"file_data_json: {file_data_json}")
                            update_json(file_data_json, data_type, modify, total)
                            logger.debug(f"after update json")
                            logger.debug(f"{file_data_json} updated")
                        exit_code = 3
                    else:
                        logger.debug(f"input file :{inputfile} 不包含{FILE_ORIGIN_DATA_XLSX.split('.')[0]}/{FILE_SEG_EFFECT_XLSX.split('.')[0]}/{FILE_WORD_CHOICE_MATCH_XLSX.split('.')[0]}的字段，未导入！！！")
                        exit_code = 104 
            logger.info(f"导入数据结束")
        # 聚类
        elif args.scale and args.category and args.task and (args.file_type in (1,2,3)):
            logger.info(f"开始聚类")
            if args.file_type == 1:
                logger.info(f"使用原始数据表")
                cluster_srcfilename_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_ORIGIN_DATA_XLSX)
                cluster_srcfilename_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_ORIGIN_DATA_CSV)
            elif args.file_type == 2:
                logger.info(f"使用分词效果表")
                cluster_srcfilename_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_SEG_EFFECT_XLSX)
                cluster_srcfilename_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_SEG_EFFECT_CSV)
            elif args.file_type == 3:
                logger.info(f"使用选词匹配表")
                cluster_srcfilename_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_WORD_CHOICE_MATCH_XLSX)
                cluster_srcfilename_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW , FILE_WORD_CHOICE_MATCH_CSV)
            file_data_json = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON)
            file_cluster_json = os.path.join(gooseeker_tag_local , args.task , FILE_CLUSTER_JSON)
            logger.debug(f"cluster_srcfilename_xlsx：{cluster_srcfilename_xlsx}")
            #data_cluster(cluster_srcfilename_xlsx, args.file_type, args.scale, args.category, file_data_json, file_cluster_json)
            # 获取vectorize和聚类参数
            para_dict = get_para_dict(os.path.join(gooseeker_tag_local , args.task , 'cluster'), args.file_type, args.scale, args.category)
            logger.info(f"聚类参数：{para_dict}")
            # 聚类
            cluster_dir = os.path.join(gooseeker_tag_local , args.task ,DIR_CLUSTER)
            exit_code = int(data_cluster(cluster_srcfilename_xlsx, args.file_type, args.scale, args.category, file_data_json, file_cluster_json, para_dict, cluster_dir))
            logger.info(f"聚类完成")
        # LDA
        elif args.topic and args.model and args.task and args.scale and (args.file_type in (1,2,3)): 
            logger.info(f"lda开始")
            if args.file_type == 1:
                logger.info(f"使用原始数据表")
                lda_srcfilename_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_ORIGIN_DATA_XLSX)
                lda_srcfilename_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_ORIGIN_DATA_CSV)
            elif args.file_type == 2:
                logger.info(f"使用分词效果表")
                lda_srcfilename_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_SEG_EFFECT_XLSX)
                lda_srcfilename_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_SEG_EFFECT_CSV)
            elif args.file_type == 3:
                logger.info(f"使用选词匹配表")
                lda_srcfilename_xlsx = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_WORD_CHOICE_MATCH_XLSX)
                lda_srcfilename_csv = os.path.join(gooseeker_tag_local , args.task , DIR_RAW_LDA , FILE_WORD_CHOICE_MATCH_CSV)
            file_data_json_lda = os.path.join(gooseeker_tag_local , args.task , FILE_DATA_JSON_LDA)
            file_lda_json = os.path.join(gooseeker_tag_local , args.task , FILE_LDA_JSON)
            logger.debug(f"lda_srcfilename_xlsx：{lda_srcfilename_xlsx}")
            # 获取vectorize和聚类参数
            lda_dir = os.path.join(gooseeker_tag_local , args.task ,DIR_LDA)
            para_dict = get_para_dict_lda(lda_dir, args.file_type, args.scale, args.topic, args.model)
            logger.info(f"LDA参数：{para_dict}")
            # lda
            exit_code = int(data_lda(lda_srcfilename_xlsx, args.file_type, args.scale, args.topic , args.model , args.task, file_data_json_lda, file_lda_json, para_dict, lda_dir))
        else:
            logger.info(f"未提供聚类或者LDA或者导入数据需要的参数")
        logger.debug(f"finished .....")
    except Exception as e:
        #logger.error('异常信息:' + str(e))
        logger.error(f"{traceback.format_exc()}")
        exit_code = 103 
    logger.debug(f"exit_code: {exit_code}")
    sys.exit(exit_code)

"""
exit_code:
正常情况：
0   正常退出
-f 4的情况下：1.原始数据 2.分词效果表 3.选词匹配表
4   聚类正常结束
错误或异常情况：
101 input文件不存在
102 input文件类型不支持
103 有其它异常发生
104 excel表不包含指定字段
105 聚类源文件不存在
106 聚类源数据无有效数据
107 lda/nmf model参数无效
108 lda/nmf 保存图片失败
"""




